@@ -24,12 +24,14 @@ seq.IDate <-
 c.IDate <-
 rep.IDate <-
 split.IDate <-
-as.list.IDate <-
 unique.IDate <-
     function(x, ...) {
         as.IDate(NextMethod())
     }
 
+# fix for #1315
+as.list.IDate <- function(x, ...) NextMethod()
+
 # rounding -- good for graphing / subsetting
 ## round.IDate <- function (x, digits, units=digits, ...) {
 ##     if (missing(digits)) digits <- units # workaround to provide a units argument to match the round generic and round.POSIXt
@@ -72,16 +74,20 @@ as.ITime.POSIXlt <- function(x, ...) {
 
 as.character.ITime <- format.ITime <- function(x, ...) {
     # adapted from chron's format.times
-    x <- unclass(x)
-    hh <- x %/% 3600
-    mm <- (x - hh * 3600) %/% 60
-    ss <- trunc(x - hh * 3600 - 60 * mm)
-    paste(substring(paste("0", hh, sep = ""), nchar(paste(hh))), 
-          substring(paste("0", mm, sep = ""), nchar(paste(mm))), 
-          substring(paste("0", ss, sep = ""), nchar(paste(ss))), sep = ":")
+    # Fix for #811. Thanks to @StefanFritsch for the code snippet
+    neg <- x < 0L
+    x  <- abs(unclass(x))
+    hh <- x %/% 3600L
+    mm <- (x - hh * 3600L) %/% 60L
+    ss <- trunc(x - hh * 3600L - 60L * mm)
+    res = paste(substring(paste("0", hh, sep = ""), nchar(paste(hh))), 
+              substring(paste("0", mm, sep = ""), nchar(paste(mm))), 
+              substring(paste("0", ss, sep = ""), nchar(paste(ss))), sep = ":")
+    if (any(neg)) res[neg] = paste("-", res[neg], sep="")
+    res
 }
 
-as.data.frame.ITime = function(x, ...) {
+as.data.frame.ITime <- function(x, ...) {
     # This method is just for ggplot2, #1713
     # Avoids the error "cannot coerce class '"ITime"' into a data.frame", but for some reason
     # ggplot2 doesn't seem to call the print method to get axis labels, so still prints integers.
