@@ -1,10 +1,10 @@
 
-test.data.table = function(verbose=FALSE, pkg="pkg") {
+test.data.table <- function(verbose=FALSE, pkg="pkg") {
     if (exists("test.data.table",.GlobalEnv,inherits=FALSE)) {
         # package developer
         if ("package:data.table" %in% search()) stop("data.table package loaded")
         if (.Platform$OS.type == "unix" && Sys.info()['sysname'] != "Darwin")
-            d = path.expand("~/R/gitdatatable/pkg/inst/tests")
+            d = path.expand("~/data.table/inst/tests")
         else {
             if (!pkg %in% dir()) stop(paste(pkg, " not in dir()", sep=""))
             d = paste(getwd(),"/", pkg, "/inst/tests",sep="")
@@ -45,12 +45,12 @@ test.data.table = function(verbose=FALSE, pkg="pkg") {
 # whichfail = NULL
 # .devtesting = TRUE
 
-compactprint = function(DT, topn=2) {
+compactprint <- function(DT, topn=2) {
     cn = paste(" [Key=",paste(key(DT),collapse=",")," Types=",paste(substring(gsub("integer64","i64",sapply(DT,class)),1,3),collapse=","),"]",sep="")
     print(copy(DT)[,(cn):=""], topn=topn)
     invisible()
 }
-test = function(num,x,y,error=NULL,warning=NULL,output=NULL) {
+test <- function(num,x,y,error=NULL,warning=NULL,output=NULL) {
     # Usage:
     # i) tests that x equals y when both x and y are supplied, the most common usage
     # ii) tests that x is TRUE when y isn't supplied
@@ -70,15 +70,32 @@ test = function(num,x,y,error=NULL,warning=NULL,output=NULL) {
     assign("ntest", get("ntest", parent.frame()) + 1, parent.frame(), inherits=TRUE)   # bump number of tests run
     assign("lastnum", num, parent.frame(), inherits=TRUE)
     v = getOption("datatable.verbose")
-    d = exists(".devtesting",parent.frame()) && get(".devtesting", parent.frame())
-    if (v || d) {
-        cat(if (v && !d) "\n\n" else "\r", "Running test id ", num, "     ",sep="")
+    i = interactive()   # exists(".devtesting",parent.frame()) && get(".devtesting", parent.frame())
+    if (v || i) {
+        cat(if (i) "\r" else "\n\n", "Running test id ", num, "     ",sep="")
     }
     # TO DO: every line that could possibly fail should ideally be inside test()
     xsub = substitute(x)
     ysub = substitute(y)
     if (is.null(output)) err <<- try(x,TRUE)
     else out = gsub("NULL$","",paste(capture.output(print(err<<-try(x,TRUE))),collapse=""))
+    if (!is.null(output)) {
+        output = gsub("[[]","<LBRACKET>",output)
+        output = gsub("[]]","<RBRACKET>",output)
+        output = gsub("<LBRACKET>","[[]",output)
+        output = gsub("<RBRACKET>","[]]",output)
+        output = gsub("[(]","[(]",output)
+        output = gsub("[)]","[)]",output)
+        if (!length(grep(output,out))) {
+            cat("Test",num,"didn't produce correct output:\n")
+            cat(">",deparse(xsub),"\n")
+            cat("Expected: '",output,"'\n",sep="")
+            cat("Observed: '",out,"'\n",sep="")
+            assign("nfail", nfail+1, parent.frame(), inherits=TRUE)
+            assign("whichfail", c(whichfail, num), parent.frame(), inherits=TRUE)
+            return()
+        }
+    }
     if (!is.null(error) || !is.null(warning)) {
         type = ifelse(!is.null(error),"error","warning")
         patt = txt = ifelse(!is.null(error),error,warning)
@@ -110,25 +127,8 @@ test = function(num,x,y,error=NULL,warning=NULL,output=NULL) {
         assign("whichfail", c(whichfail, num), parent.frame(), inherits=TRUE)
         return()
     }
-    if (!is.null(output)) {
-        output = gsub("[[]","<LBRACKET>",output)
-        output = gsub("[]]","<RBRACKET>",output)
-        output = gsub("<LBRACKET>","[[]",output)
-        output = gsub("<RBRACKET>","[]]",output)
-        output = gsub("[(]","[(]",output)
-        output = gsub("[)]","[)]",output)
-        if (!length(grep(output,out))) {
-            cat("Test",num,"didn't produce correct output:\n")
-            cat(">",deparse(xsub),"\n")
-            cat("Expected: '",output,"'\n",sep="")
-            cat("Observed: '",out,"'\n",sep="")
-            assign("nfail", nfail+1, parent.frame(), inherits=TRUE)
-            assign("whichfail", c(whichfail, num), parent.frame(), inherits=TRUE)
-            return()
-        }
-        if (missing(y)) return()
-    }
     if (missing(y)) {
+        if (!is.null(output)) return()
         if (isTRUE(as.vector(x))) return()  # as.vector to drop names of a named vector such as returned by system.time
         cat("Test",num,"expected TRUE but observed:\n")
         cat(">",deparse(xsub),"\n")
