@@ -21,7 +21,18 @@ myfunction(DT,sum(Sepal.Width))
 
 
 ###################################################
-### code chunk number 3: datatable-faq.Rnw:145-152
+### code chunk number 3: datatable-faq.Rnw:109-115
+###################################################
+DT = as.data.table(iris)
+whatToRun = quote( .(AvgWidth = mean(Sepal.Width),
+                     MaxLength = max(Sepal.Length)) )
+DT[, eval(whatToRun), by=Species]
+DT[, eval(whatToRun), by=.(FirstLetter=substring(Species,1,1))]
+DT[, eval(whatToRun), by=.(Petal.Width=round(Petal.Width,0))]
+
+
+###################################################
+### code chunk number 4: datatable-faq.Rnw:158-165
 ###################################################
 X = data.table(grp=c("a","a","b","b","b","c","c"), foo=1:7)
 setkey(X,grp)
@@ -33,7 +44,7 @@ X[Y,sum(foo*bar),by=.EACHI]
 
 
 ###################################################
-### code chunk number 4: datatable-faq.Rnw:180-183
+### code chunk number 5: datatable-faq.Rnw:193-196
 ###################################################
 DF = data.frame(x=1:3,y=4:6,z=7:9)
 DF
@@ -41,20 +52,20 @@ DF[,c("y","z")]
 
 
 ###################################################
-### code chunk number 5: datatable-faq.Rnw:186-188
+### code chunk number 6: datatable-faq.Rnw:199-201
 ###################################################
 DT = data.table(DF)
 DT[,c(y,z)]
 
 
 ###################################################
-### code chunk number 6: datatable-faq.Rnw:191-192
+### code chunk number 7: datatable-faq.Rnw:204-205
 ###################################################
 DT[,.(y,z)]
 
 
 ###################################################
-### code chunk number 7: datatable-faq.Rnw:201-207
+### code chunk number 8: datatable-faq.Rnw:214-220
 ###################################################
 data.table(NULL)
 data.frame(NULL)
@@ -65,7 +76,7 @@ is.null(data.frame(NULL))
 
 
 ###################################################
-### code chunk number 8: datatable-faq.Rnw:211-214
+### code chunk number 9: datatable-faq.Rnw:224-227
 ###################################################
 DT = data.table(a=1:3,b=c(4,5,6),d=c(7L,8L,9L))
 DT[0]
@@ -73,7 +84,7 @@ sapply(DT[0],class)
 
 
 ###################################################
-### code chunk number 9: datatable-faq.Rnw:236-239
+### code chunk number 10: datatable-faq.Rnw:249-252
 ###################################################
 DT = data.table(x=rep(c("a","b"),c(2,3)),y=1:5)
 DT
@@ -81,7 +92,7 @@ DT[,{z=sum(y);z+3},by=x]
 
 
 ###################################################
-### code chunk number 10: datatable-faq.Rnw:245-250
+### code chunk number 11: datatable-faq.Rnw:258-263
 ###################################################
 DT[,{
   cat("Objects:",paste(objects(),collapse=","),"\n")
@@ -91,27 +102,27 @@ DT[,{
 
 
 ###################################################
-### code chunk number 11: datatable-faq.Rnw:256-258
+### code chunk number 12: datatable-faq.Rnw:269-271
 ###################################################
 DT[,.(g=1,h=2,i=3,j=4,repeatgroupname=x,sum(y)),by=x]
 DT[,.(g=1,h=2,i=3,j=4,repeatgroupname=x[1],sum(y)),by=x]
 
 
 ###################################################
-### code chunk number 12: datatable-faq.Rnw:277-279
+### code chunk number 13: datatable-faq.Rnw:290-292
 ###################################################
 A = matrix(1:12,nrow=4)
 A
 
 
 ###################################################
-### code chunk number 13: datatable-faq.Rnw:282-283
+### code chunk number 14: datatable-faq.Rnw:295-296
 ###################################################
 A[c(1,3),c(2,3)]
 
 
 ###################################################
-### code chunk number 14: datatable-faq.Rnw:290-293
+### code chunk number 15: datatable-faq.Rnw:303-306
 ###################################################
 B = cbind(c(1,3),c(2,3))
 B
@@ -119,7 +130,7 @@ A[B]
 
 
 ###################################################
-### code chunk number 15: datatable-faq.Rnw:296-301
+### code chunk number 16: datatable-faq.Rnw:309-314
 ###################################################
 rownames(A) = letters[1:4]
 colnames(A) = LETTERS[1:3]
@@ -129,7 +140,7 @@ A[B]
 
 
 ###################################################
-### code chunk number 16: datatable-faq.Rnw:304-309
+### code chunk number 17: datatable-faq.Rnw:317-322
 ###################################################
 A = data.frame(A=1:4,B=letters[11:14],C=pi*1:4)
 rownames(A) = letters[1:4]
@@ -139,20 +150,20 @@ A[B]
 
 
 ###################################################
-### code chunk number 17: datatable-faq.Rnw:312-314
+### code chunk number 18: datatable-faq.Rnw:325-327
 ###################################################
 B = data.frame(c("a","c"),c("B","C"))
 cat(try(A[B],silent=TRUE))
 
 
 ###################################################
-### code chunk number 18: datatable-faq.Rnw:393-394
+### code chunk number 19: datatable-faq.Rnw:406-407
 ###################################################
 base::cbind.data.frame
 
 
 ###################################################
-### code chunk number 19: datatable-faq.Rnw:401-404
+### code chunk number 20: datatable-faq.Rnw:414-417
 ###################################################
 foo = data.frame(a=1:3)
 cbind.data.frame = function(...)cat("Not printed\n")
@@ -160,13 +171,13 @@ cbind(foo)
 
 
 ###################################################
-### code chunk number 20: datatable-faq.Rnw:406-407
+### code chunk number 21: datatable-faq.Rnw:419-420
 ###################################################
 rm("cbind.data.frame")
 
 
 ###################################################
-### code chunk number 21: datatable-faq.Rnw:461-467
+### code chunk number 22: datatable-faq.Rnw:474-480
 ###################################################
 DT = data.table(a=rep(1:3,1:3),b=1:6,c=7:12)
 DT
@@ -177,7 +188,7 @@ DT[,{ mySD = copy(.SD)
 
 
 ###################################################
-### code chunk number 22: datatable-faq.Rnw:473-479
+### code chunk number 23: datatable-faq.Rnw:486-492
 ###################################################
 DT = data.table(a=c(1,1,2,2,2),b=c(1,2,2,2,1))
 DT
@@ -188,7 +199,7 @@ cat(try(
 
 
 ###################################################
-### code chunk number 23: datatable-faq.Rnw:483-489
+### code chunk number 24: datatable-faq.Rnw:496-502
 ###################################################
 if (packageVersion("data.table") >= "1.8.1") {
     DT[,.N,by=list(a,b)][,unique(N),by=a]
@@ -199,7 +210,7 @@ if (packageVersion("data.table") >= "1.9.3") {
 
 
 ###################################################
-### code chunk number 24: datatable-faq.Rnw:507-515
+### code chunk number 25: datatable-faq.Rnw:520-528
 ###################################################
 DT = data.table(a=1:5,b=1:5)
 suppressWarnings(
